/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: riadiaci modul urceny, k prechadzaniu suborov v zadanom adresari a tvorbe prislusnych vystupov do podobnej adresarovej struktury
 *        obsahuje funkciu main 
 *        vyuziva triedy: MelEn_transform  
 *   
 */


#include <iostream>
#include <string.h>
#include <fstream>
#include <dirent.h>
#include <cstring>
#include <algorithm>
#include <sys/stat.h>
#include <unistd.h>

#include "melEn_transform.h"

using namespace std;

/**
 * pomocna struktura uchovavajuca informacie o parametroch prikazoveho riadku
 */ 
struct params {
   int i_help;          // indikuje vyskyt parametru pre napovedu
   int i_coef;          // indikuje zadanie poctu Mel energii v jednom vektore
   int coef;            // pocet Mel energii vednom vektore, ktore subory obsahuju
   int i_addr;          // indikuje zadanie vstupneho adresara so subormi obsahujucimi Mel energie
   int i_out;           // indikuje zadanie vystupneho suboru, kde sa maju priznakove vektory ulozit
   char *addr;          // vstupny adresar
   char *out;           // vystupny subor
};

/**
 * inicializuje hodnoty struktury params
 * @param param odkaz na vstupnu strukturu 
 */
void initParam(struct params *param) {
    
    param->i_help = 0;
    param->i_coef = 0;
    param->coef = 0;
    param->i_addr = 0;
    param->i_out = 0;
    param->addr = NULL;
    param->out = NULL;
}

/**
 * nacitanie parametrov z prikazovej riadky a kontrola ich spravneho formatu a kombinacie
 */
int procArgs(int argc, char *argv[], struct params *param) {

    int opt=0;
    
    //extern int optind,opterr,optop;
    while ((opt=getopt(argc,argv,"hc:i:o:"))!=-1) {
        switch (opt) {
            case 'h':
                param->i_help++;
                break;
            case 'c':
                param->i_coef++;
                param->coef = atoi(optarg);
                break;
            case 'i': 
                param->i_addr++;
                param->addr = optarg;
                break;
            case 'o':
                param->i_out++;
                param->out = optarg;
                break;
            case '?':
                printf("Pre napovedu skuste parameter -h\n");
                return -1;
                break;
            default:
                printf("Pre napovedu skuste parameter -h\n");
                return -1;
                break;
        }
    }
       
    if ((param->i_help > 1) || (param->i_coef > 1) || (param->i_addr > 1) || (param->i_out > 1)) {
        return -2;
    }
    
    if (param->i_help) {
        if (param->i_coef || param->i_addr) {
            return -3;
        }
        return 1;
    }
    
    if (param->i_coef == 0) {
        return -4;
    }

    if (param->i_addr == 0) {
        return -4;
    }
    
    if (param->i_out == 0) {
        return -4;
    }
    
    return 0;

}


/*
 * napoveda
 */
void napoveda() {

   cout <<
"Nastroj make_PNG - vytvara z Mel energii PGM obrazky\n\
 Autor: Kamil Behun\n\
 verzia: 2\n\
 - tento program bol vytvoreny pre ucely bakalarskej prace\n\
\n\
Program urceny k vytvaraniu obrazkov v PGM formate z Mel energii spektra.\n\
 Vytvori sa tak obrazok vizualizujuci spektrogram prislusnej (pesnicky), ktoremu\n\
zodovedaju Mel energie vstupneho suboru. Pred vytvorenim spektrogramu su Mel energie\n\
kvoli znizeniu dynamickeho rozsahu prevedene na Logaritmice Mel energie spektra\n\
pomocou vzorca 'log10(koeficient + 1)/maxlog*255'.\n\
\n\
Program prechadza rekurzivne vstupny adresar a vyladava subory s Mel energiami,\n\
vytvorene obrazky k tymto suborom uklada do zadaneho adresara, kde vytvori rovnaku\n\
strukturu ako vo vstupnom a obrazok ulozi na prislusne misto zodpovedajuce pozicii\n\
vstupnemu suboru s Mel energiami.\n\
\n\
Program je mozne spustit s nasledovnymi parametrami:\n\
\n\
     -h           Napoveda\n\
\n\
     -c cislo     Udava kolko Mel energii sa v jednom vektore (na jednom riadku)\n\
                  vo vstupnych suboroch nachadza\n\
\n\
     -i adresar   Vstupny adresar obsahujuci subory s Mel koeficientami pre prislusne\n\
                  zanre (zaner je reprezentovany adresarom).\n\
\n\
     -o adresar   Vystupny adresar kam sa ukladaju vytvorene obrazky zodpovedajuce\n\
                  vstupom.\n\
\n";
}


/**
 * prechadza zadanu adresarovu strukturu a vytvara ke nej prislusne vystupy
 * @param input - nazov vstupneho adresara
 * @param output - nazov vystupneho adresara
 * @param ft - obejkt triedy MelEn_transform, pomocou ktoreho sa vytvara prislusny vystup v pozadovanom formate zo vstupneho suboru
 * @return chybova hodnota    
 */ 
int proc_addr(const char *input, const char *output, MelEn_transform & ft) {

    DIR *pDIR;
    struct dirent *entry;
    if( (pDIR=opendir(input)) != NULL ) {    // otvorenie zadaneho adreara

        mkdir(output,0777);   // vytvorenie prislusneho vystupneho adresara

        while((entry = readdir(pDIR)) != NULL){     // citanie vstupneho adresara
                if( strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0 ) {
                  string s1_in(input);
                  string s1_out(output);
                  string s2("/");
                  string s3(entry->d_name);
                  s1_in += s2 + s3;
                  s1_out += s2 + s3;
                  proc_addr(s1_in.c_str(), s1_out.c_str(), ft);     // rekurzivne volanie
                }
        }
        closedir(pDIR);
    }
    else {    // ide o subor

        string file(input);
        int pos = file.find(".flt");

        if (pos >= 0 && (file.length() == (unsigned int)pos + 4)) { // kontrola ci je to subor s mel koeficientami
           string out_file(output);
           pos = out_file.find(".flt");
           out_file.replace(out_file.begin() + pos, out_file.end(), ".pgm");

           if (ft.make_picture(input,out_file.c_str()) != 0) {  // vytvorenie vystupu zo vstupneho suboru
              cout << "Chyba pri vytvarani vyskupneho obrazka PGM s nazovom: " << out_file.c_str() << endl;
           }
           ft.clear_max_log();
        }
    }
    
    return 0;

}



/***************************************
 *            Funkcia main             *
 *           --------------            *
 ***************************************/    
int main(int argc, char *argv[]) {

    params pr;
    
    initParam(&pr);
    
    if (procArgs(argc, argv, &pr) < 0) {
        cout << "chyba zadanych parametrov, pre napovedu zadajte: -h" << endl;
        return 1;
    }

    if (pr.i_help) {
       napoveda();
       return 0;
    }

    MelEn_transform ft(pr.coef);
    
    proc_addr(pr.addr, pr.out, ft);

    return 0;
}