/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: implementacia metod triedy MelEn_transform 
 *   
 */

#include <fstream>
#include <iostream>
#include <math.h>


#include "melEn_transform.h"


/**
 * vytvori PGM vystup zo vstupneho suboru obsahujuceho mel energie spektra, predtym ich vsak prevede na Logaritmicke Mel energie spektra
 * @param input - nazov vstupneho suboru
 * @param output - nazov vystupneho suboru
 * @return chybova hodnota   
 */ 
int MelEn_transform::make_picture(const char *input, const char *output) {

    ofstream fOUT(output);
    ifstream fIN(input);
    
    if (!fIN) {
        return -1;
    }
    
    int lines = count_lines_find_max(fIN);
    int counter = 0;
    int len_max = num_length(max_output_val);
    
    float hlp_num;
    int i, out_num;
    
    fOUT << "P2" << endl;
    fOUT << "# " << input << endl;
    fOUT << coef << " " << lines << endl;
    fOUT << max_output_val << endl;
    
    while (fIN >> hlp_num) {
    
        out_num = scheme(hlp_num);
    
        for (i = 0; i < len_max - num_length(out_num); i++) {
            fOUT << " ";
        }
    
        fOUT << out_num;
        
        counter++;
        if (counter >= coef) {
            fOUT << endl;
            counter = 0;
        }
        else {
            fOUT << " ";
        }
    }
    
    return 0;
}

/**
 * spocita pocet riadkov vstupneho suboru a najde maximalny logaritmus Mel energie v subore
 * @param fIN - handler vstupneho suboru
 * @return chybova hodnota  
 */ 
int MelEn_transform::count_lines_find_max(ifstream & fIN) {
    
    if (!fIN) {
        return -1;
    }
    int i = 0;
    int count = 0;
    float hlp;
    
    while (fIN >> hlp) {
        count++;
	check_max_log(hlp);
        if (count >= coef) {
            count = 0;
            i++;
        }
    }
   
    fIN.clear(); 
    fIN.seekg(0,ios::beg);
    
    return i;
    
}

/**
 * skontroluje, ci logaritmus zadaneho cisla nie je maximum a ak ano maximum zmeni
 * @param nmb - kontrolovane cislo
 */
void MelEn_transform::check_max_log(float nmb) {

    double lg = log10(nmb  + 1);

    if (lg > max_log_val) {
	max_log_val = lg;
    }
}

/**
 * prevedie vstupnu Mel energiu na Logaritmicku Mel energiu spektra
 * @param nmb - prevadzane cislo
 * @return vystupna hodnota  
 */ 
int MelEn_transform::scheme(float nmb) {

    int pom = (int)(((log10(nmb + 1) / max_log_val) * max_output_val));
    /*if (pom < 0) {
        return 0;
    } */
    return pom;
}

/**
 * vracia cislo urcujuce pocet cisiel z kolkych sa vstupne cislo sklada
 * @param nmb - vstupne cislo
 * @return pocet cisiel, z ktorych sa vstupne cislo sklada  
 */ 
int MelEn_transform::num_length(int nmb) {

    int len = 1;
    nmb /= 10;
    while (nmb > 0) {
        len++;
        nmb /= 10;
    }
    
    return len;
}

/**
 * vymazanie maximalnej logaritmickej hodnoty Mel energie v subore
 */
void MelEn_transform::clear_max_log() {

   max_log_val = -std::numeric_limits<double>::max();

}
