/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: implementacia metod triedy Histogram
 *   
 */

#include "histogram.h"
#include <iostream>
#include <fstream>
#include <cfloat>

using namespace std;

/**
 * konstruktor triedy Histogram
 * @param n_secv - pocet intervalov hitogramu
 * @param maximum - maximalna hodnota histogramu
 * @param minimum - minimalna hodnota histogramu   
 */ 
Histogram::Histogram(int n_secv, float maximum, float minimum) {

    secv = n_secv;
    max = maximum;
    min = minimum;
    for (int i = 0; i < secv; i++) {
        hst[i] = 0;    
    }

}

/**
 * zaraduje zadane cislo do prislusneho intervalu histogramu
 * @param num - cislo, ktore sa pokusae zaradit do histogramu 
 */ 
void Histogram::insert(float num) {

    float segment = (max - min) / (float) secv;

    for (int i = 0; i < secv; i++) {
        if (i + 1 >= secv) {
            if ((num >= (min +(segment * i))) && (num <= max)) {
                hst[i]++;
            }
        }
        else {
            if ((num >= (min +(segment * i))) && (num <= (min +(segment * (i + 1))))) {
                hst[i]++;
                break;
            }
        }
    }
}

/**
 * vyprazdni histogram, cim sa mysli vynuluje jednotlive hodnoty v intervaloch histogramu
 */ 
void Histogram::clear(void) {
    
    for (int i = 0; i < secv; i++) {
        hst[i] = 0;
    }

}

/**
 * vrati histogram, ktory trieda uklada v podobe pola hodnot
 * @return histogram v podobe pola hodnot 
 */ 
map<int,double> & Histogram::get_hist(void) {
    return hst;
}

/**
 * prevadza konkatenaciu dvoch histogramov a vysledok uklada v prislusnej instancii tejto triedy, ktora operaciu vola
 * @param h - druhy histogram, s ktorym chceme konkatenaciu vykonat 
 */ 
void Histogram::concatenate_1(map<int,double> & h) {

    for (unsigned int i = 0; i < hst.size(); i++) {
        if (h[i] < hst[i]) {
            hst[i] = h[i];
        }
    }
    /*cout << "coef: ";
    for (unsigned int i = 0; i < h.size(); i++) {
        cout << h[i] << " ";
    }
    cout << endl;*/
}

/**
 * metoda, ktora uklada histogram do zadaneho suboru vo formate LibSVM s prislusnou triedou (labelom)
 * @param label - urcuje triedu
 * @param fOUT - handler vystupneho suboru  
 */ 
void Histogram::print_to_file(int begin, ofstream & fOUT) {

    for (unsigned int i = 0; i < hst.size(); i++) {
        fOUT << " " << begin + i << ":" << hst[i]; 
    }
}

/**
 * sluzi k vypisaniu obsahu triedy
 * vypise na vystup histogram, cim sa mysli jedntlive intervali a k nim aj prislusne hodnoty 
 */ 
void Histogram::vypis(void) {

    float segment = (max - min) / (float) secv;

    for (int i = 0; i < secv; i++) {
        if (i + 1 >= secv) {
            cout << i << ":: < " << (min +(segment * i)) << " ; " << max << " > = " << hst[i] << endl;
            
        }
        else {
            cout << i << ":: < " << (min +(segment * i)) << " ; " << (min +(segment * (i + 1))) << " > = " << hst[i] << endl;
        }
    }
}

/**
 * pomocna metoda na zjednoduseny vypis histogramu
 * vypisuju sa len hodnoty, nie intervaly 
 */ 
void Histogram::skuska(void) {
    
    for (unsigned int i = 0; i < hst.size(); i++) {
        cout << i << ": " << hst[i] << endl;
    
    }

}

/**
 * sluzi na normalizaciu hodnot histogramu
 * vacsinou sa zadava pocet vektorov MFCC, ktore sa do histogramu ukladali
 * @param opt - hodnota, ktorou sa histogram normalizuje  
 */ 
void Histogram::optimization(int opt) {

    for (unsigned int i = 0; i < hst.size(); i++) {
        hst[i] /= (double) opt;    
    }
}
