/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: (riadiaci) modul urceny primarne pre funkciu main a prislusne riadiace funkcie programu 
 *   
 */


#include <stdio.h>
#include <cstdlib>
#include <iostream>
#include <string.h>
#include <cstring>
#include <fstream>
#include <dirent.h>
#include <unistd.h>
#include <getopt.h>

#include "histogram.h"
#include "mfcc_hist.h"
#include "max_min.h"

using namespace std;

/**
 * pomocna struktura uchovavajuca informacie o parametroch prikazoveho riadku
 */ 
struct parametre {
   int i_help;          // indikuje vyskyt parametru pre napovedu
   int i_coef;          // indikuje zadanie poctu MFCC v jednom vektore
   int coef;            // pocet koeficientov v jednom vektore, ktore subory obsahuju
   int i_seg;           // indikuje zadanie poctu segmetov, ktore ma histogram obsahovat
   int seg;             // pocet segmetov histogramu
   int i_addr_train;    // indikuje zadanie vstupneho adresara trenovacich dat so subormi obsahujucimi vektory MFCC
   int i_addr_test;     // indikuje zadanie vstupneho adresara testovacich dat so subormi obsahujucimi vektory MFCC
   int i_out_train;     // indikuje zadanie vystupneho suboru pre trenovacie data, kde sa maju priznakove vektory ulozit
   int i_out_test;      // indikuje zadanie vystupneho suboru pre testovacie data, kde sa maju priznakove vektory ulozit
   int saveMaxMin;      // indikuje zadanie xml suboru do ktoreho sa ulozia maxima a minima jednotlivych koeficientov
   int loadMaxMin;      // indikuje zadanie xml suboru z ktoreho sa nacitaju maxima a minima jednotlivych koeficientov
   int findMaxMin;      // indikuje, ze sa ma vyhladat maximum a minimum zo zadanych vstupnych adresarov
   int makeFeatures;    // indikuje, ze sa maju vytvorit histogramy pre jednotlive koeficienty a z nich nasledne vektor priznakov
   char *xmlFileMaxMin; // nazov xml suboru
   char *addr_train;    // vstupny adresar pre trenovacie data
   char *out_train;     // vystupny subor pre trenovacie data
   char *addr_test ;    // vstupny adresar pre testovacie data
   char *out_test;      // vystupny subor pre testovacie data
};

struct option longopts[] = {
   { "help",          no_argument,       NULL,         'h' },
   { "findMaxMin",    no_argument,       NULL,         'f' },
   { "makeFeatures",  no_argument,       NULL,         'k' },
   { "saveMaxMin",    required_argument, NULL,         'm' },
   { "loadMaxMin",    required_argument, NULL,         'l' },
   { "inTestData",    required_argument, NULL,         't' },
   { "inTrainData",   required_argument, NULL,         'i' },
   { "outTestFile",   required_argument, NULL,         'v' },
   { "outTrainFile",  required_argument, NULL,         'o' },
   { 0, 0, 0, 0 }
};


/**
 * inicializuje hodnoty v strukture
 * @param param odkaz na vstupnu strukturu 
 */
void initParam(struct parametre *param) {
    
    param->i_help = 0;
    param->i_coef = 0;
    param->coef = 0;
    param->i_seg = 0;
    param->seg = 0;
    param->i_addr_train = 0;
    param->i_addr_test = 0;
    param->i_out_train = 0;
    param->i_out_test = 0;
    param->saveMaxMin = 0;
    param->loadMaxMin = 0;
    param->findMaxMin = 0;
    param->makeFeatures = 0;
    param->xmlFileMaxMin = NULL;
    param->addr_train = NULL;
    param->addr_test = NULL;
    param->out_train = NULL;
    param->out_test =NULL;
}

/**
 * vypise hodnoty jednotlivych parametrov zadanych v prikazovom riadku
 */
void vypis_parametre(struct parametre *param) {

    cout << "help: " << param->i_help << endl;
    cout << "koeficienty: " << param->i_coef << " ,pocet: " << param->coef << endl;
    cout << "segmenty: " << param->i_seg << " ,pocet: " << param->seg << endl;
    cout << "findMaxMin: " << param->findMaxMin << endl;
    cout << "makeFeatures: " << param->makeFeatures << endl;
    if (param->i_addr_train)
       cout << "vstupny adresar pre trenovacie data: " << param->i_addr_train << " ,nazov: " << param->addr_train << endl;
    if (param->i_addr_test)
       cout << "vstupny adresar pre testovacie data: " << param->i_addr_test << " ,nazov: " << param->addr_test << endl;
    if (param->i_out_train)
       cout << "vystupny adresar pre trenovacie data: " << param->i_out_train << " ,nazov: " << param->out_train << endl;
    if (param->i_out_test)
       cout << "vystupny adresar pre testovacie data: " << param->i_out_test << " ,nazov: " << param->out_test << endl;
    if (param->saveMaxMin)
       cout << "ulozit maxmin: " << param->saveMaxMin << " ,nazov: " << param->xmlFileMaxMin << endl;
    if (param->loadMaxMin)
       cout << "nacitat maxmin: " << param->loadMaxMin << " ,nazov: " << param->xmlFileMaxMin << endl;
}

/**
 * nacitanie parametrov z prikazovej riadky a kontrola ich sprevneho zadania
 */
int procArgs(int argc, char *argv[], struct parametre *param) {

    int opt=0;

    while ((opt=getopt_long(argc,argv,"hc:s:i:o:", longopts, NULL))!=-1) {
        switch (opt) {
            case 'h':
                param->i_help++;
                break;
            case 'f':
                param->findMaxMin++;
                break;
            case 'k':
                param->makeFeatures++;
                break;
            case 'c':
                param->i_coef++;
                param->coef = atoi(optarg);
                break;
            case 's':
                param->i_seg++;
                param->seg = atoi(optarg);
                break;
            case 'i': 
                param->i_addr_train++;
                param->addr_train = optarg;
                break;
            case 't': 
                param->i_addr_test++;
                param->addr_test = optarg;
                break;
            case 'o':
                param->i_out_train++;
                param->out_train = optarg;
                break;
            case 'v':
                param->i_out_test++;
                param->out_test = optarg;
                break;
            case 'm':
                param->saveMaxMin++;
                param->xmlFileMaxMin = optarg;
                break;
            case 'l':
                param->loadMaxMin++;
                param->xmlFileMaxMin = optarg;
                break;
            case '?':
                printf("Pre napovedu skuste parameter -h\n");
                return -1;
                break;
            default:
                printf("Pre napovedu skuste parameter -h\n");
                return -1;
                break;
        }
    }
       
    if ((param->i_help > 1) || (param->i_coef > 1) || (param->i_seg > 1) || (param->i_addr_train > 1) || (param->i_out_train > 1) 
        || (param->saveMaxMin > 1) || (param->loadMaxMin > 1) || (param->i_addr_test > 1) || (param->i_out_test > 1) || (param->findMaxMin > 1)
        || (param->makeFeatures > 1)) {
        return -2;
    }
    
    if (param->i_help) {
        if (param->i_coef || param->i_seg || param->i_addr_train || param->i_addr_test || param->i_out_train || param->i_out_test
            || param->loadMaxMin || param->saveMaxMin || param->findMaxMin || param->makeFeatures) {
            return -3;
        }
        return 1;
    }
    
    if (param->i_coef == 0) {
        return -4;
    }
    
    if (param->i_seg == 0) {
        param->seg = 16;
    }
    
    if (param->i_addr_train == 0 && param->i_addr_test == 0) {
        return -4;
    }
    
    if (param->makeFeatures && param->i_addr_train && param->i_out_train == 0) {
        return -4;
    }

    if (param->makeFeatures && param->i_addr_test && param->i_out_test == 0) {
        return -4;
    }

    return 0;

}

/*
 * napoveda
 */
void napoveda() {

   cout <<
"Nastroj make_hist - vytvara MFCC histogramy a z tychto histogramov priznakovy vektor\n\
Autor: Kamil Behun\n\
verzia: 3\n\
- tento nastroj bol vytvoreny pre ucely bakalarskej prace\n\
\n\
Program make_hist je urceny k vytvaraniu priznakovych vektorov zo suboru vektorov MFCC\n\
pomocou vytvarania histogramov MFCC. Pre vytvaranie histogramov MFCC je nutne ziskat\n\
jednotlive rozsahy a tak umoznuje make_hist tieto rozsahy vyhladat zo vstupnych súborov\n\
MFCC, prípadne tieto rozsahy nacitat z XML suboru alebo naopak tieto rozsahy do XML suboru\n\
ulozit.\n\
\n\
Pri vytvarani jednotlivych histogramov MFCC tieto rozsahy nasledne vuzije. Po vytvoreni\n\
histogramov pre jednotlive MFCC, tieto histogramy make_hist prevedie na vystupny vektor\n\
priznakov skonkatenovanim. Vysledny vektor priznakov bude vo formate LibSVM a label bude\n\
urceny v zavislosti na prislusnom zanri, respektive adresari v akom sa nachadza subor,\n\
pre ktory sa histogramy MFCC momentalne vytvarali. Program make_hist vygeneruje pre kazdu\n\
vstupnu adresarovu strukturu vygeneruje textovy subor obsahujuci cisla labelov\n\
pre jednotlive zanre.\n\
\n\
Program make_hist teda prechadza zadanu adresarovu strukturu a postupne pre kazdy subor\n\
MFCC (.mfc) vytvara histogramy MFCC a z nich prislusny vektor priznakov vo formate LibSVM.\n\
Tieto vektory priznakov uklada do vystupneho suboru, ktory bude obsahovat vsetky vektory\n\
priznakov jednotlivych MFCC suborov pre zadanu vstupnu adresarovu strukturu.\n\
Program make_hist umoznuje spracovat MFCC subory pre trenovaciu a nevidenu sadu zaroven\n\
\n\
Proram je spustany s nasledovnymi parametrami:\n\
\n\
     --help|-h              Napoveda\n\
\n\
     --findMaxMin|-f        Parameter, ktory zabezpeci vyhladanie rozsahov jednotlivych\n\
                            MFCC pre subory MFCC zadanych vstupnych adresarovych struktur\n\
\n\
     --makeFeatures|-k      Parameter, ktory zabespeci vetvorenie vystupnych suborov\n\
                            vektorov priznakov pre MFCC subory zadanych vstupnych\n\
                            adresarovych struktur\n\
\n\
     --saveMaxMin=nazov     Urcuje nazov XML suboru kam sa rozsahy jednotlivych MFCC ulozia\n\
\n\
     --loadMaxMin=nazov     Urcuje nazob XML suboru, z ktoreho sa maju rozsahy jednotlivych\n\
                            MFCC nacitat. Mozno ich nacitat a zaroven aj aktualizovat \n\
                            vyhladanim rozsahov v novych vstupnych adresarovych strukturach.\n\
\n\
     --inTrainData=nazov    Zadanie vstupnej adresarovej struktury obsahujucu MFCC subory\n\
                            pesniciek pre jednotlive zanre reprezentovane adresarmi\n\
                            pre trenovaciu sadu.\n\
\n\
     --inTestData=nazov     Zadanie vstupnej adresarovej struktury obsahujucu MFCC subory\n\
                            pesniciek pre jednotlive zanre reprezentovane adresarmi\n\
                            pre nevidenu sadu.\n\
\n\
     --outTrainFile=nazov   Zadanie nazvu vystupneho suboru kam sa budu ukladat priznakove\n\
                            vektory pre MFCC subory trenovacej sady.\n\
\n\
     --outTestFile=nazov    Zadanie nazvu vystupneho suboru kam sa budu ukladat priznakove\n\
                            vektory pre MFCC subory nevidenej sady.\n\
\n";
} 

/**
 * spracovanie jedneho adresara reprezentujuceho jeden zaner
 * adresar obsahuje subory s vektormi MFCC prislusnych pesniciek
 * @param gen_addr - nazov adresara obsahujuceho subory s MFCC jedneho zanru
 * @param hst_gnr - odkaz na instanciu triedy obsahujucu histogramy,
 *                  ktora bude po prevedeni funkcie obsahovat skonkatenovane histogrami jednotlivych suborov v tomto adresari
 * @param fOUT - handler vystupneho suboru kams a budu priznakove vektory ukladat
 * @param label - je cislo, ktore bude reprezentovat zaner v priznakovych vektorov
 * @return chybova hodnota
 */ 
int proc_one_genre(const char *gen_addr, Mfcc_hist *hst_gnr, ofstream & fOUT, int label) {

    int pos;
    DIR *genDIR;
    struct dirent *entry;
    
    if ((genDIR = opendir(gen_addr)) == NULL) {
        return -1;
    }
    
    while ((entry = readdir(genDIR)) != NULL) {
        if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
            string s3(entry->d_name);
            pos = s3.find(".mfc");
            if (pos >= 0 && (s3.length() == (unsigned int)pos + 4)) {
                string s1(gen_addr);
                string s2("/");
                s1 += s2 + s3;
                if (hst_gnr->make_histograms(s1.c_str()) < 0) {
                    return -2;
                }
                hst_gnr->print_to_file(label,fOUT);
                hst_gnr->clear_histograms();
            }
        }
    }

    return 0;
}

/**
 * funkcia urcena k separacii jednotlivych adresarov reprezentujucich jednotlive zanre
 * dalej vola funkciu pre spracovane prislusneho zanra a pre tento zaner
 * generuje textovy subor, ktory obsahuje cisla labelov pre jedntlive zanre
 * @param param - odkaz na strukturu obsahujucu informacie o parametroch prikazoveho riadku
 * @param m - intsancia obsahujucu zaznam o maximach a minimach jednotlivych MFCC
 * @param in_addr - urcuje nazov vstupnej adresarovej struktury
 * @param out_file - urcuje nazov vystupneho suboru kam sa budu ukladat priznakove vektory
 * @param label_numbers - nazov suboru kam sa budu ukladat cisla labelov pre jednotlive zanre
 * @return chybova hodnota
 */ 
int read_genres(struct parametre *param, Max_min & m, const char *in_addr, const char *out_file, const char *label_numbers) {

    DIR *pDIR;
    int rtr;
    ofstream fOUT(out_file);
    ofstream fOUT_lab(label_numbers); 
    Mfcc_hist hst_gnr(m,param->coef,param->seg);
    int label = 1;

    if (fOUT == NULL || fOUT_lab == NULL) {
        return -2;
    }

    if ((pDIR = opendir(in_addr)) == NULL) {
        return -1;
    }

    struct dirent *entry;
    while ((entry = readdir(pDIR)) != NULL) {
        if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
            string s1(in_addr);
            string s2("/");
            string s3(entry->d_name);
            s1 += s2 + s3;

            if ((rtr = proc_one_genre(s1.c_str(), &hst_gnr, fOUT, label)) == 0) {
                fOUT_lab << entry->d_name << ": " << label << endl;
                label++;
            }
            else {
                if (rtr < 1) {
                    return -3;
                }
            }

            hst_gnr.clear_histograms();
        }
    }

    return 0;

}

/**** ******** ****** ********** *********
 *        HLAVNA FUNKCIA MAIN            *
 *       --------------------            *
 **** ***************************** ******/  
int main(int argc, char *argv[]){

    struct parametre param;
    int rtr;

    initParam(&param);
    if ((rtr = procArgs(argc,argv,&param)) < 0) {
        cout << "chyba pri zadavani parametrov parametrov, skuste napovedu -h\n";
        return 1;
    }

    if (param.i_help) {
       napoveda();
       return 0;
    }

    vypis_parametre(&param);

    Max_min m(param.coef);

    if (param.loadMaxMin) {
       cout << "nacitavam Maxima a Minima zo suboru: " << param.xmlFileMaxMin << endl;
       m.load_from_file(param.xmlFileMaxMin);
    }

    if (param.findMaxMin) {
        if (param.i_addr_train) {
           cout << "hladam maxima a minima z korenoveho adresara: " << param.addr_train << endl;
           if (m.find_MxMn(param.addr_train) < 0) {
              cout << "find_MxMn: chyba pri ovarani suboru alebo adresara pre trenovacie data\n";
           }
        }
        if (param.i_addr_test) {
           cout << "hladam maxima a minima z korenoveho adresara: " << param.addr_test << endl;
           if (m.find_MxMn(param.addr_test) < 0) {
              cout << "find_MxMn: chyba pri ovarani suboru alebo adresara pre testovacie data\n";
           }
        }
    }

    if (param.saveMaxMin) {
       cout << "ukladam Maxima a Minima do suboru: " << param.xmlFileMaxMin << endl;
       m.save_to_file(param.xmlFileMaxMin);
    }

    //m.save_to_file(param.xmlFileMaxMin);
    //m.load_from_file(param.xmlFileMaxMin);
    m.vypis();

    if (param.makeFeatures) {
       if (param.i_addr_train) {
          cout << "vytvaram subor s vektormi priznakov pre korenovy adresar: " << param.addr_train << endl;
          if (read_genres(&param,m,param.addr_train,param.out_train, "train_labels.txt") < 0) {
             cout << "read_genres: chyba pri otvarani suboru alebo adresara pre trenovacie data\n";
          } 
       }

       if (param.i_addr_test) {
          cout << "vytvaram subor s vektormi priznakov pre korenovy adresar: " << param.addr_test << endl;
          if (read_genres(&param,m,param.addr_test,param.out_test, "test_labels.txt") < 0) {
             cout << "read_genres: chyba pri otvarani suboru alebo adresara pre testovacie data\n";
          } 
       }
    }

    return 0;
}
