/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: implementacia metod triedy Max_min
 *   
 */ 
 
#include "max_min.h"

#include <cfloat>
#include <iostream>
#include <string.h>
#include <fstream>
#include <dirent.h>
#include <cstring>


/**
 * konstruktor triedy Max_min
 * nastavi premenne triedy na pociatocne hodnoty 
 * @param n_coef - pocet cepstralnych koeficientov 
 */  
Max_min::Max_min(int n_coef) {

    num_coef = n_coef;

    for (int i = 0; i < n_coef; i++) {
        maximum[i] = -FLT_MAX;
        minimum[i] = FLT_MAX;
    }
}

/**
 * rekurzivne prechadza zadanu adresarovu strukturu
 * a hlada vsetky ".mfc" subory (subory obsahujuce vektory MFCC)
 * @param addr - adresarova struktura obsahujuca subory s MFCC
 * @return chybova hodnota   
 */ 
int Max_min::find_MxMn(const char* addr) {
    
    DIR *pDIR;
    struct dirent *entry;
    if( (pDIR=opendir(addr)) != NULL ) {
        while((entry = readdir(pDIR)) != NULL){
                if( strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0 ) {
                  string s1(addr);
                  string s2("/");
                  string s3(entry->d_name);
                  s1 += s2 + s3;
                  cout << s1.c_str() << endl;
                  find_MxMn(s1.c_str());
                }
        }
        closedir(pDIR);
    }
    else {
        //cout << addr << endl;
        string file(addr);
        int pos = file.find(".mfc");

        if (pos >= 0 && (file.length() == (unsigned int)pos + 4)) {
           if (proc_file(addr) < 0) {
              return -1;
           }
        }
    }

    return 0;
}

/**
 * prechazda zadany subor, tak aby vyhladal maximalne a minimalne hodnoty MFCC
 * @param file - nazov suboru
 * @return chybova hodnota  
 */ 
int Max_min::proc_file(const char *file) {

    float number;
    //printf("%s\n",file);
    ifstream F(file);
    if (!F) {
        return -1;
    }
    
    int count = 0;
    
    while (F >> number) {
        if (count >= num_coef)
           count = 0;
        check_MxMn(count,number);
        count++;
    }
    
    return 0;
}

/**
 * kontrola, ci zadana hodnota nie je nove maximum alebo minimum zadaneho koeficientu
 * @param count - urcuje kontorolovany koeficient
 * @param num - kontrolovana hodnota  
 */ 
void Max_min::check_MxMn(int count, float num) {

    if (maximum[count] < num) {
        maximum[count] = num;
    }
    
    if (minimum[count] > num) {
        minimum[count] = num;
    }

}

/**
 * vracia maximumalnu hodnotu zadaneho MFCC
 * @param index - urcuje koeficient
 * @return maximalna hodnota pre zadany koeficient  
 */ 
float Max_min::ret_max(int index) {

    if (index < num_coef && index >= 0) {
        return maximum[index];
    }
    
    return -FLT_MAX;
}

/**
 * vracia minimalnu hodnotu zadaneho MFCC
 * @param index - urcuje koeficient
 * @return minimalna hodnota  
 */ 
float Max_min::ret_min(int index) {

    if (index < num_coef && index >= 0) {
        return minimum[index];
    }
    
    return FLT_MAX;
}

/**
 * vypise vsetky maxima a minima vsetkych MFCC na standartny vystup
 */ 
void Max_min::vypis(void) {

    for (int i = 0; i < num_coef; i++) {
        cout << i << ":: max: " << maximum[i] << ", min: " << minimum[i] << endl;
    }
}


/**
 * uklada minima a maxima jednotlivych koeficientov do XML suboru
 * @param file_name - meno suboru, do ktoreho sa parametre ulozia
 */
void Max_min::save_to_file(const char *file_name) {

   TiXmlDocument doc;
   TiXmlDeclaration * decl = new TiXmlDeclaration( "1.0", "", "" );
   doc.LinkEndChild( decl );

   TiXmlElement * element = new TiXmlElement( "maxmin" );
   element->SetAttribute("coef", num_coef);
   doc.LinkEndChild(element);

   for (int i = 0; i < num_coef; i++) {
       TiXmlElement * new_cf = new TiXmlElement( "coefficient" );
       new_cf->SetAttribute("index",i);
       new_cf->SetDoubleAttribute("max", maximum[i]);
       new_cf->SetDoubleAttribute("min", minimum[i]);
       element->LinkEndChild(new_cf);
   }

   doc.SaveFile( file_name );

}


/**
 * nacita z xml suboru maxima a minima pre jednotlive koeficienty
 * @param file_name - nazov xml suboru
 * @return chybova hodnota, 0 vsetko prebehlo v poriadku
 */
int Max_min::load_from_file(const char *file_name) {

   TiXmlDocument doc(file_name);
   if (!doc.LoadFile()) {
      return -1;
   }

   TiXmlHandle hDoc(&doc);
   TiXmlElement* pElem;
   TiXmlHandle hRoot(0);
   int pocet_coef = 0;

   // block: maxmin

   pElem=hDoc.FirstChildElement().Element();

   if (!pElem) {
      return -2;
   }

   if (strcmp("maxmin",pElem->Value()) != 0) {
      return -3;
   }

   pElem->QueryIntAttribute("coef", &pocet_coef);

   if (pocet_coef != num_coef) {
      return -4;
   }

   // save this for later
   hRoot=TiXmlHandle(pElem);

   // block: coefficient

   float mn;
   float mx;
   int index;

   pElem=hRoot.FirstChild( "coefficient" ).Element();
   for( ; pElem; pElem=pElem->NextSiblingElement()) {
      pElem->QueryIntAttribute("index",&index);
      pElem->QueryFloatAttribute("max",&mx);
      pElem->QueryFloatAttribute("min",&mn);
      maximum[index] = mx;
      minimum[index] = mn;
   }

   return 0;
}

