/* 
 * autor: Kamil Behun
 * email: xbehun03@stud.fit.vutbr.cz 
 * popis: implementacia metod triedy Mfcc_hist
 *   
 */

#include <iostream>
#include <fstream>

#include "mfcc_hist.h"
#include "max_min.h"


using namespace std;

/**
 * konstruktor triedy Mfcc_hist
 * @param extr - objekt triedy Max_min obsahujuci maximalne a minimalne hodnoty jednotlivych MFCC
 * @param num_coef - pocet cepstralnych koeficientov
 * @param num_arr - pocet pozadovanych intervalov histogramov jednotlivych ceps. koeficientov   
 */ 
Mfcc_hist::Mfcc_hist(Max_min & extr, int num_coef, int num_arr) {

    num_segs = num_arr;

    for (int i = 0; i < num_coef; i++) {
        coef[i] = new Histogram(num_arr,extr.ret_max(i),extr.ret_min(i));
    }


}

/**
 * metoda urcena pre vytvorenie histogramov pre koeficienty zadaneho suboru
 * @param file - nazov suboru pre ktory sa bude histogram vytvarat. Subor obsahuje vektory MFCC
 * @return chybova hodnota  
 */ 
int Mfcc_hist::make_histograms(const char *file) {

    float number;
    ifstream F(file);
    if (!F) {
        return -1;
    }
    
    unsigned int count = 0;
    int count_lines = 0;
    
    while (F >> number) {
        if (count >= coef.size()) {
           count = 0;
        }
        coef[count]->insert(number);
        count++;
        if (count == coef.size()) {
            count_lines++;
        }
    }
    
    // optimalizacia histogramov poctom vektorov koeficientov
    for (unsigned int i = 0; i < coef.size(); i++) {
        coef[i]->optimization(count_lines);
    }
    cout << "pocet riadkov: " << count_lines << endl;
    coef[0]->vypis();
    
    return 0;
}

/**
 * vyprazdnenie histogramov jednotlivych koeficientov
 */ 
void Mfcc_hist::clear_histograms(void) {

    for (unsigned int i = 0; i < coef.size(); i++) {
        coef[i]->clear();
    }
}

/**
 * vracia pole obsahujuce histogram zadaneho koeficientu
 * @param index - urcuje koeficient, ktoreho chceme histogram
 * @return pole obsahujuce histogram  
 */ 
map<int,double> & Mfcc_hist::get_hist(unsigned int index) {
    return coef[index]->get_hist();
}

/**
 * zkonkatenuje histogramy jednotlivych koeficientov pre dva subory
 * @param h - instancia obsahujuca histogramy koeficientov pre jeden subor 
 */ 
void Mfcc_hist::concatenate(Mfcc_hist & h) {
    
    for (unsigned int i = 0; i < coef.size(); i++) {
        coef[i]->concatenate_1(h.get_hist(i));
    }
}

/**
 * vytlaci obsah histogramov do zadaneho suboru spolu s prislusnou triedou
 * vystup bude vo formate LibSVM  (pre kazdy histogram sa vola prislusna metoda)
 * @param label - prislusna trieda
 * @param fOUT - handler vystupneho suboru  
 */ 
void Mfcc_hist::print_to_file(int label,ofstream & fOUT) {
    
    fOUT << label;
    
    for (unsigned int i = 0; i < coef.size(); i++) {
        coef[i]->print_to_file(num_segs * i + 1, fOUT);
    }
    
    fOUT << endl; 
}

/**
 * destruktor uvolnujuci pamat vyuzitu pre jednotlive histogramy koeficientov
 */ 
Mfcc_hist::~Mfcc_hist() {

    for (unsigned int i = 0; i < coef.size(); i++) {
        delete coef[i];
    }
    
}